/* \file vec2.cpp
 *
 */
#include "stdafx.h"
#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include <iostream>
#include "vec2.h"

//! overloaded insertion operator <<
std::ostream& operator << (std::ostream& s, const Vec2& v)
{
	s << "[" << v[0] << " "  << v[1] << "]";
	return s ;
}

//! overloaded extraction operator >>
std::istream& operator >> (std::istream& s, Vec2& v) {
	char	c = ' ';

	while (isspace(c))
		s >> c;
    // The vectors can be formatted either as x y or [ x y ]
	if (c == '[') {
		s >> v[0] >> v[1];
		while (s >> c && isspace(c)) ;
		if (c != ']') s.putback(c); // let someone else deal with it
	}
	else {
		s.putback(c);
		s >> v[0] >> v[1];
	}
	return s;
}

//! return magnitude of cross product
/*!
 *  for 2D this is dot(perp(a),b)\n
 *  for 3D this is the magnitude of cross(a,b)
 */
double perpdot(const Vec2 &a, const Vec2 &b)
{
	return a[0]*b[1]-a[1]*b[0];
}

//! returns angle (in degrees) between two vectors
double angle(const Vec2 &a, const Vec2 &b)
{
	double c, s;
	const double dgr = 45.0/atan(1.0);
	c = dot(a,b);
	s = perpdot(a,b);
	return dgr*atan2(s,c);
}

const double radg = atan(1.0)/45.0;


Vec2::Vec2(double theta)
{
	theta *= radg;
	set(cos(theta),sin(theta));
}

//! rotate Vec2 about the origin by angle theta in degrees
Vec2 &Vec2::rotate(double theta)
{
	double c, s;
	Vec2 &a = *this;
	if (theta==0.0) return a;
	theta *= radg;
	c = cos(theta);
	s = sin(theta);
	double tmp = s*a[0]+c*a[1];
	a[0] = c*a[0]-s*a[1];
	a[1] = tmp;
	return a;
}

//! returns vec rotated by angle theta in degrees
Vec2 rotate(const Vec2 &a, double theta)
{
	double c, s;
	theta *= radg;
	c = cos(theta);
	s = sin(theta);
	return Vec2(c*a[0]-s*a[1],s*a[0]+c*a[1]);
}
